/*
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contributor(s): Blender Foundation (2008).
 *
 * ***** END GPL LICENSE BLOCK *****
 */

/* note: the original vertex color stuff is now just used for
 * getting info on the layers themselves, accessing the data is
 * done through the (not yet written) mpoly interfaces.*/

/** \file blender/makesrna/intern/rna_mesh.c
 *  \ingroup RNA
 */

#include <stdlib.h>

#include "MEM_guardedalloc.h"

#include "DNA_material_types.h"
#include "DNA_mesh_types.h"
#include "DNA_meshdata_types.h"
#include "DNA_object_types.h"

#include "BLI_math_base.h"
#include "BLI_math_rotation.h"
#include "BLI_utildefines.h"

#include "BKE_editmesh.h"

#include "RNA_access.h"
#include "RNA_define.h"
#include "RNA_types.h"

#include "rna_internal.h"

#include "WM_types.h"

EnumPropertyItem mesh_delimit_mode_items[] = {
	{BMO_DELIM_NORMAL, "NORMAL", 0, "Normal", "Delimit by face directions"},
	{BMO_DELIM_MATERIAL, "MATERIAL", 0, "Material", "Delimit by face material"},
	{BMO_DELIM_SEAM, "SEAM", 0, "Seam", "Delimit by edge seams"},
	{0, NULL, 0, NULL, NULL},
};

#ifdef RNA_RUNTIME

#include "DNA_scene_types.h"

#include "BLI_math.h"

#include "BKE_customdata.h"
#include "BKE_depsgraph.h"
#include "BKE_main.h"
#include "BKE_mesh.h"
#include "BKE_report.h"

#include "ED_mesh.h" /* XXX Bad level call */

#include "WM_api.h"
	return me;
}

static CustomData *rna_mesh_pdata_helper(Mesh *me)
{
	return (me->edit_btmesh) ? &me->edit_btmesh->bm->pdata : &me->pdata;
